#!/bin/sh
### ====================================================================
###  @UNIX-shell-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "0.10",
###     date            = "04 December 1998",
###     time            = "11:00:59 MST",
###     filename        = "biborder.sh",
###     address         = "Center for Scientific Computing
###                        University of Utah
###                        Department of Mathematics, 322 INSCC
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 585 1640, +1 801 581 4148",
###     checksum        = "56789 870 3019 25412",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@ieee.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "bibliography, ordering, BibTeX",
###     supported       = "yes",
###     docstring       = "This file contains the biborder utility, a
###                        program for arranging key/value pairs into
###                        a standard order in BibTeX data base files.
###
###                        Indentation is preserved on all lines, but
###                        trailing whitespace is discarded.  Entries
###                        themselves, and any comments present
###                        between entries, are NOT reordered.  The
###                        companion utility, bibsort(1), provides
###                        that service.
###
###                        Syntax errors in the input stream will
###                        cause abrupt termination with a fatal error
###                        message and a non-zero exit code.  The
###                        output will be incomplete, so you should
###                        always verify file line counts (input and
###                        output counts should be identical) before
###                        assuming that you can replace the input
###                        file with the output file.  If nawk is
###                        used, biborder will verify input and output
###                        line counts when it terminates normally at
###                        end of file; it cannot do so with gawk.
###
###                        Usage:
###                              biborder
###                                     [-alphabetical]
###					[-bimonthly m1/m2/m3/m4/m5/m6 ]
###					[-check-missing]
###					[-monthly]
###					[-quarterly m1/m2/m3/m4 ]
###					bibfile(s) >outfile
###                        or
###                              biborder
###                                     [-alphabetical]
###					[-bimonthly m1/m2/m3/m4/m5/m6 ]
###					[-check-missing]
###					[-monthly]
###					[-quarterly m1/m2/m3/m4 ]
###					<infile >outfile
###
###			   Switch names may be abbreviated to the
###			   minimal unique prefix.
###
###                        If -alphabetical is specified, the standard
###                        order of keys is ignored, and strict
###                        alphabetical order is used instead.
###
###			   If -check-missing is specified, missing
###			   expected fields will be supplied, with the
###			   field name prefixed with OPT, and the value
###			   string set to a pair of question marks,
###			   e.g.  OPTvolume = "??".  The OPT prefix
###			   ensures that the key is ignored by BibTeX
###			   (and thus that the question marks will not
###			   appear in an output .bbl file), and
###			   together with the question marks,
###			   highlights the missing data.  In addition,
###			   the GNU Emacs bibtex-mode editing support
###			   has convenient functions for removing the
###			   OPT prefixes, and so does bibclean(1).
###
###			   If -bimonthly is specified, all of the
###			   journals in the BibTeX file entries are
###			   assumed to appear at two-month
###			   intervals. The six month names that
###			   correspond to each such interval must be
###			   given as a SINGLE argument following
###			   -bimonthly, with the names separated by
###			   (any number of) non-letters.  A typical
###			   example is
###
###			   feb-apr-jun-aug-oct-dec
###
###			   specifying that issue number 1 appears in
###			   February, and number 6 in December.
###
###			   If -monthly is specified, all of the
###			   journals in the BibTeX file entries are
###			   assumed to appear monthly, 1 in January, 2
###			   in February, ..., 12 in December.  If only
###			   one of the number and month values is
###			   supplied, biborder will supply the other
###			   automatically, but will spell the key name
###			   in uppercase letters to distinguish it (a
###			   simple editor string substitution, or
###			   bibclean(1), can subsequently normalize the
###			   appearance).  If both month and number are
###			   specified in a BibTeX entry, they are
###			   checked for consistency, and a warning is
###			   issued if they do not match.
###
###			   If -quarterly is specified, all of the
###			   journals in the BibTeX file entries are
###			   assumed to appear quarterly. The four month
###			   names that correspond to each quarter must
###			   be given as a SINGLE argument following
###			   -quarterly, with the names separated by
###			   (any number of) non-letters.  Some examples
###			   are
###
###				   jan-apr-jul-oct
###				   mar////jun.sep:dec
###				   Spring\ Summer\ Fall\ Winter
###				   'Spring Summer Fall Winter'
###
###			   The last two examples show that when space
###			   separators are used, they must be
###			   backslashed, or else the list must be
###			   surrounded by quotes, to force UNIX shells
###			   to interpret the quarter names as a single
###			   argument.
###
###			   The -bimonthly, -monthly, and -quarterly
###			   options are mutually exclusive, and should
###			   only be used in journal-specific
###			   bibliographies, since number-month pairing
###			   does not hold for all journals.
###
###                        All remaining command-line words are assumed
###                        to be input files.  Should such a filename
###                        begin with a hyphen, it must be disguised by
###                        a leading absolute or relative directory
###                        path, e.g. /tmp/-foo.bib or ./-foo.bib.
###
###                        WARNINGS:
###
###                        (1) This simple version does NOT recognize
###                        bib entries with outer parentheses instead of
###                        braces, or with line breaks between the @Name
###                        and following opening brace.  Use bibclean(1)
###                        to standardize and syntax check the
###                        bibliography entries first.
###
###                        (2) Implementation limitations in nawk or
###                        gawk may result in premature termination
###                        because of maximum string lengths being
###                        exceeded.  This can happen with long
###                        abstract or summary strings.  This problem
###                        has been seen more frequently with some
###                        UNIX nawk implementations than with the
###                        Free Software Foundation GNU Project's
###                        gawk, so we prefer to use gawk, if
###                        available.
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
########################################################################

# Assign default initial values
ALPHABETICAL=0
FILES=
BIMONTHLY=0
CHECKMISSING=0
MONTHLY=0
QUARTERLY=0

# Loop over the command-line arguments, collecting biborder switches,
# and file names.
while [ $# -gt -0 ]
do
	case $1 in
	-a*)
		ALPHABETICAL=1
		;;
	-b*)
		BIMONTHLY=1
		shift
		MONTHNAMES="$1"
		;;
	-c*)
		CHECKMISSING=1
		;;
	-m*)
		MONTHLY=1
		;;
	-q*)
		QUARTERLY=1
		shift
		MONTHNAMES="$1"
		;;
	-*)
		echo Usage: $0
		echo '		[-bimonthly m1/m2/m3/m4/m5/m6]'
		echo '		[-check-missing]'
		echo '		[-monthly]'
		echo '		[-quarterly m1/m2/m3/m4]'
		echo '		BibTeXfiles or <infile'
		echo '		>outfile'
		exit 1
		;;
	*)			# everything else is assumed to be a filename
		FILES="$FILES $1"
		;;
	esac
	shift			# discard this switch or filename
done

# Use GNU gawk instead of nawk: Sun Solaris 2.x nawk often complains
# `input record too long'.
mawk \
	-v ALPHABETICAL=$ALPHABETICAL \
	-v BIMONTHLY=$BIMONTHLY \
	-v CHECKMISSING=$CHECKMISSING \
	-v MONTHLY=$MONTHLY \
	-v QUARTERLY=$QUARTERLY \
	-v MONTHNAMES="$MONTHNAMES" \
	-f `dirname $0`/biborder.awk \
	$FILES
################################[The End]###############################
